*===============================================================;
*  Title:  PROGRAM A1 ASSIGNS AHRQ PREVENTION QUALITY INDICATORS
*          TO INPATIENT RECORDS
*
*  Description:;
*         ASSIGNS THE OUTCOME OF INTEREST FLAGS TO INPATIENT
*         RECORDS;
*
*          >>>  VERSION 3.2, March 2008 <<<
*
*  USER NOTE:  Make sure you have created format library 
*              (using PQFMTS.SAS) BEFORE running this program.
*
*===============================================================;
FILENAME CONTROL 'C:\PATHNAME\CONTROL_PQI.SAS'; *<===USER MUST modify;

%INCLUDE CONTROL;

 TITLE2 'PROGRAM A1';
 TITLE3 'AHRQ PREVENTION QUALITY INDICATORS: ASSIGN QIS TO INPATIENT DATA';

 * -------------------------------------------------------------- ;
 * --- PREVENTION QUALITY INDICATOR (PQI) NAMING CONVENTION:  --- ;
 * --- THE FIRST LETTER IDENTIFIES THE PREVENTION QUALITY     --- ;
 * --- INDICATOR AS ONE OF THE FOLLOWING:
                (O) OBSERVED RATES
                (E) EXPECTED RATES
                (R) RISK-ADJUSTED RATES
                (S) SMOOTHED RATES
                (T) NUMERATOR ("TOP")
                (P) POPULATION ("POP")
 * --- THE SECOND LETTER REFERS TO THE INDICATOR              --- ;
 * --- SUBTYPE, (A)MBULATORY CARE SENSITIVE CONDITION. THE    --- ;
 * --- NEXT TWO CHARACTERS ARE ALWAYS 'PQ'. THE LAST TWO      --- ;
 * --- DIGITS ARE THE INDICATOR NUMBER (WITHIN THAT SUBTYPE). --- ;
 * -------------------------------------------------------------- ;

 %MACRO CTY2MA;
    %IF &MALEVL EQ 0 %THEN %DO;
        ATTRIB MAREA LENGTH=$5
          LABEL='FIPS STATE COUNTY CODE';
        MAREA = FIPSTCO;
    %END;
    %ELSE %IF &MALEVL EQ 1 %THEN %DO;
        ATTRIB MAREA LENGTH=$5
          LABEL='MODIFIED FIPS';
        MAREA = INPUT(PUT(FIPSTCO,$M1AREA.),$5.);
    %END;
    %ELSE %IF &MALEVL EQ 2 %THEN %DO;
        ATTRIB MAREA LENGTH=$5
          LABEL='OMB 1999 METRO AREA';
        MAREA = INPUT(PUT(FIPSTCO,$M2AREA.),$5.);
    %END;
    %ELSE %IF &MALEVL EQ 3 %THEN %DO;
        ATTRIB MAREA LENGTH=$5
          LABEL='OMB 2003 METRO AREA';
        MAREA = INPUT(PUT(FIPSTCO,$M3AREA.),$5.);
    %END;
 %MEND;

 %MACRO MDX(FMT);

 (%DO I = 1 %TO &NDX.-1;
  (PUT(DX&I.,&FMT.) = '1') OR
  %END;
  (PUT(DX&NDX.,&FMT.) = '1'))

 %MEND;

 %MACRO MDX1(FMT);

 ((PUT(DX1,&FMT.) = '1'))

 %MEND;

 %MACRO MDX2(FMT);

 (%DO I = 2 %TO &NDX.-1;
  (PUT(DX&I.,&FMT.) = '1') OR
  %END;
  (PUT(DX&NDX.,&FMT.) = '1'))

 %MEND;

 %MACRO MPR(FMT);

 (%DO I = 1 %TO &NPR.-1;
  (PUT(PR&I.,&FMT.) = '1') OR
  %END;
  (PUT(PR&NPR.,&FMT.) = '1'))

 %MEND;


DATA OUT1.&OUTFILA1.
                 (KEEP=KEY DRG MDC AGE MAREA FIPST YEAR DQTR 
                       AGECAT POPCAT SEXCAT RACECAT AGECCAT 
                       TAPQ01--TAPQ16 TAPQ90-TAPQ92);
 SET IN1.&INFILE1.
                 (KEEP=KEY DRG MDC SEX AGE AGEDAY PSTCO 
                       ASOURCE ATYPE RACE YEAR DQTR 
                       DX1-DX&NDX. PR1-PR&NPR.);
                       
                       
 * -------------------------------------------------------------- ;
 * --- DIAGNOSIS AND PROCEDURE MACROS --------------------------- ;
 * -------------------------------------------------------------- ;

 * -------------------------------------------------------------- ;
 * --- DELETE RECORDS WITH MISSING VALUES FOR AGE OR SEX--------- ;
 * -------------------------------------------------------------- ;
 IF SEX LE 0 THEN DELETE;
 IF AGE LT 0 THEN DELETE;
 
 * -------------------------------------------------------------- ;
 * --- DEFINE FIPS STATE AND COUNTY CODES             ----------- ;
 * -------------------------------------------------------------- ;
 ATTRIB FIPSTCO LENGTH=$5
   LABEL='FIPS STATE COUNTY CODE';
 FIPSTCO = PUT(PSTCO,Z5.);

 ATTRIB FIPST LENGTH=$2
   LABEL='STATE FIPS CODE';
 FIPST = SUBSTR(FIPSTCO,1,2);

 * -------------------------------------------------------------- ;
 * --- CONDITIONALLY DEFINE METRO AREAS (MA) FROM COUNTY CODES -- ;
 * -------------------------------------------------------------- ;
 %CTY2MA

 * -------------------------------------------------------------- ;
 * --- MS-DRG                                                  -- ;
 * -------------------------------------------------------------- ;
 IF YEAR IN (2008) OR (YEAR IN (2007) AND DQTR IN (4)) THEN
    DRG = PUT(PUT(DRG,Z3.),$MSDRG.);

 * -------------------------------------------------------------- ;
 * --- DEFINE STRATIFIER: RACE CATEGORY ------------------------- ;
 * -------------------------------------------------------------- ;
 ATTRIB RACECAT LENGTH=3
   LABEL='PATIENT RACE/ETHNICITY';

 SELECT (RACE);
   WHEN (1)  RACECAT = 1;
   WHEN (2)  RACECAT = 2;
   WHEN (3)  RACECAT = 3;
   WHEN (4)  RACECAT = 4;
   WHEN (5)  RACECAT = 5;
   OTHERWISE RACECAT = 6;
 END; * SELECT RACE ;

 * -------------------------------------------------------------- ;
 * --- DEFINE STRATIFIER: AGE CATEGORY  ------------------------- ;
 * -------------------------------------------------------------- ;
 ATTRIB AGECAT LENGTH=3
   LABEL='PATIENT AGE';

 SELECT;
   WHEN (      AGE < 18)  AGECAT = 0;
   WHEN (18 <= AGE < 40)  AGECAT = 1;
   WHEN (40 <= AGE < 65)  AGECAT = 2;
   WHEN (65 <= AGE < 75)  AGECAT = 3;
   WHEN (75 <= AGE     )  AGECAT = 4;
 END; * SELECT AGE ;

 * -------------------------------------------------------------- ;
 * --- DEFINE STRATIFIER: SEX CATEGORY  ------------------------- ;
 * -------------------------------------------------------------- ;
 ATTRIB SEXCAT LENGTH=3
   LABEL='PATIENT GENDER';

 SELECT (SEX);
   WHEN (1)  SEXCAT = 1;
   WHEN (2)  SEXCAT = 2;
 END; * SELECT SEX ;

 * -------------------------------------------------------------- ;
 * --- DEFINE STRATIFIER: CONDITION SPECIFIC AGE CATEGORY  ------ ;
 * -------------------------------------------------------------- ;
 ATTRIB AGECCAT LENGTH=3
   LABEL='PATIENT AGE';

 SELECT;
   WHEN (      AGE < 18)  AGECCAT = 0;
   WHEN (18 <= AGE < 45)  AGECCAT = 1;
   WHEN (45 <= AGE < 65)  AGECCAT = 2;
   WHEN (65 <= AGE < 75)  AGECCAT = 3;
   WHEN (75 <= AGE     )  AGECCAT = 4;
 END; * SELECT CONDITION SPECIFIC AGE ;

 * -------------------------------------------------------------- ;
 * --- DEFINE STRATIFIER: POPULATION CATEGORY ------------------- ;
 * -------------------------------------------------------------- ;
 ATTRIB POPCAT LENGTH=3
   LABEL='PATIENT AGE';

 POPCAT=PUT(AGE,AGEFMT.);

 * -------------------------------------------------------------- ;
 * --- DEFINE AREA LEVEL ACSC INDICATORS ------------------------ ;
 * -------------------------------------------------------------- ;

 LENGTH TAPQ01 TAPQ02 TAPQ03 TAPQ05
        TAPQ07 TAPQ08 TAPQ09 TAPQ10
        TAPQ11 TAPQ12 TAPQ13 TAPQ14 TAPQ15
        TAPQ16 TAPQ90 TAPQ91 TAPQ92 3;

 LABEL
 TAPQ01 = 'DIABETES SHORT TRM COMPLICATN (Numeratr)'
 TAPQ02 = 'PERFORATED APPENDIX           (Numeratr)'
 TAPQ03 = 'DIABETES LONG TERM COMPLICATN (Numeratr)'
 TAPQ05 = 'COPD                          (Numeratr)'
 TAPQ07 = 'HYPERTENSION                  (Numeratr)'
 TAPQ08 = 'CONGESTIVE HEART FAILURE      (Numeratr)'
 TAPQ09 = 'LOW BIRTH WEIGHT              (Numeratr)'
 TAPQ10 = 'DEHYDRATION                   (Numeratr)'
 TAPQ11 = 'BACTERIAL PNEUMONIA           (Numeratr)'
 TAPQ12 = 'URINARY INFECTION             (Numeratr)'
 TAPQ13 = 'ANGINA                        (Numeratr)'
 TAPQ14 = 'DIABETES UNCONTROLLED         (Numeratr)'
 TAPQ15 = 'ADULT ASTHMA                  (Numeratr)'
 TAPQ16 = 'LOWER EXTREMITY AMPUTATION    (Numeratr)'
 TAPQ90 = 'OVERALL PQI                   (Numeratr)'
 TAPQ91 = 'ACUTE PQI                     (Numeratr)'
 TAPQ92 = 'CHRONIC PQI                   (Numeratr)'
        ;


 * -------------------------------------------------------------- ;
 * --- IDENTIFY NEONATES AND NEWBORNS --------------------------- ;
 * -------------------------------------------------------------- ;

   NEONATE = 0;
   IF 0 <= AGEDAY <= 28 THEN NEONATE = 1;
   ELSE IF AGEDAY = . AND AGE = 0 THEN DO;
      IF ATYPE IN (4) OR %MDX($V29D.) OR %MDX($LIVEBND.)
      THEN NEONATE = 1;
   END;

   NEWBORN = 0;
   IF NEONATE THEN DO;
      IF %MDX($LIVEBND.) AND NOT AGEDAY > 0 THEN NEWBORN = 1;
      ELSE IF ATYPE IN (4) AND AGEDAY = 0 AND NOT %MDX($LIVEB2D.)
      THEN NEWBORN = 1;
   END;
 
   NORMAL = 0;
   IF NEWBORN AND DRG IN (391) THEN NORMAL = 1;

 * -------------------------------------------------------------- ;
 * --- CONSTRUCT AREA LEVEL ACSC INDICATORS --------------------- ;
 * -------------------------------------------------------------- ;

   * --- DIABETES SHORT TERM COMPLICATION  --- ;

   IF MDC NOTIN (14,15) AND (AGE >= 18) AND

      %MDX1($ACDIASD.) THEN

        TAPQ01 = 1;


   * --- PERFORATED APPENDIX               --- ;

   IF MDC NOTIN (14,15) AND (AGE >= 18) AND %MDX($ACSAP2D.) THEN DO;

      TAPQ02 = 0;

      IF %MDX($ACSAPPD.) THEN TAPQ02 = 1;

   END;


   * --- DIABETES LONG TERM COMPLICATION   --- ;

   IF MDC NOTIN (14,15) AND (AGE >= 18) AND

      %MDX1($ACDIALD.) THEN

        TAPQ03 = 1;


   * --- COPD                              --- ;

   IF MDC NOTIN (14,15) AND (AGE >= 18) AND

      (%MDX1($ACCOPDD.) OR

       (%MDX1($ACCPD2D.) AND %MDX2($ACCOPDD.))

      ) THEN

        TAPQ05 = 1;


   * --- HYPERTENSION                      --- ;

   IF MDC NOTIN (14,15) AND (AGE >= 18) AND

      %MDX1($ACSHYPD.) AND NOT %MPR($ACSCARP.) THEN

        TAPQ07 = 1;


   * --- CONGESTIVE HEART FAILURE          --- ;

   IF MDC NOTIN (14,15) AND (AGE >= 18) AND

      %MDX1($ACSCHFD.) AND NOT %MPR($ACSCARP.) THEN

      TAPQ08 = 1;


   * --- LOW BIRTH WEIGHT                  --- ;

   IF NEWBORN THEN DO;

      TAPQ09 = 0;

      IF %MDX($ACSLBWD.) THEN TAPQ09 = 1;

   END;


   * --- DEHYDRATION                       --- ;

   IF MDC NOTIN (14,15) AND (AGE >= 18) AND %MDX1($ACSDEHD.) THEN

      TAPQ10 = 1;


   * --- BACTERIAL PNEUMONIA               --- ;

   IF MDC NOTIN (14,15) AND (AGE >= 18) AND %MDX1($ACSBACD.) AND

      NOT %MDX($ACSBA2D.)  THEN

      TAPQ11 = 1;


   * --- URINARY INFECTION                 --- ;

   IF MDC NOTIN (14,15) AND (AGE >= 18) AND %MDX1($ACSUTID.) THEN

      TAPQ12 = 1;

      *** Exclude Immunocompromised state and Kidney/Urinary Tract Disorder;

      IF %MDX($IMMUNID.) OR %MPR($IMMUNIP.) OR %MDX($KIDNEY.) 
      THEN TAPQ12 = .;


   * --- ANGINA                            --- ;

   IF MDC NOTIN (14,15) AND (AGE >= 18) AND

      %MDX1($ACSANGD.) AND NOT %MPR($ACSCARP.) THEN

         TAPQ13 = 1;


   * --- DIABETES UNCONTROLLED             --- ;

   IF MDC NOTIN (14,15) AND (AGE >= 18) AND

      %MDX1($ACDIAUD.) THEN

         TAPQ14 = 1;


   * --- ADULT ASTHMA                      --- ;

   IF MDC NOTIN (14,15) AND (AGE >= 18) AND

      %MDX1($ACSASTD.) THEN

         TAPQ15 = 1;

      *** Exclude Cystic Fibrosis and Anomalies of the Respiratory System;

      IF %MDX($RESPAN.) THEN TAPQ15 = .;


   * --- LOWER EXTREMITY AMPUTATION       --- ;

   IF MDC NOTIN (14,15) AND (AGE >= 18) AND

      %MPR($ACSLEAP.) AND

      %MDX($ACSLEAD.) AND NOT %MDX($ACLEA2D.) THEN

         TAPQ16 = 1;

 * -------------------------------------------------------------- ;
 * --- CONSTRUCT AREA LEVEL COMPOSITE INDICATORS ---------------- ;
 * -------------------------------------------------------------- ;

   * --- OVERALL                          --- ;

   IF TAPQ01 = 1 OR TAPQ03 = 1 OR TAPQ05 = 1 OR TAPQ07 = 1 OR
      TAPQ08 = 1 OR TAPQ10 = 1 OR TAPQ11 = 1 OR TAPQ12 = 1 OR
      TAPQ13 = 1 OR TAPQ14 = 1 OR TAPQ15 = 1 OR TAPQ16 = 1
   THEN 
   TAPQ90 = MAX(OF TAPQ01 TAPQ03 TAPQ05 TAPQ07 
                   TAPQ08 TAPQ10 TAPQ11 TAPQ12
                   TAPQ13 TAPQ14 TAPQ15 TAPQ16);

   * --- ACUTE                            --- ;

   IF TAPQ10 = 1 OR TAPQ11 = 1 OR TAPQ12 = 1 
   THEN 
   TAPQ91 = MAX(OF TAPQ10 TAPQ11 TAPQ12);

   * --- CHRONIC                          --- ;

   IF TAPQ01 = 1 OR TAPQ03 = 1 OR TAPQ05 = 1 OR TAPQ07 = 1 OR
      TAPQ08 = 1 OR TAPQ13 = 1 OR TAPQ14 = 1 OR TAPQ15 = 1 OR
      TAPQ16 = 1
   THEN 
   TAPQ92 = MAX(OF TAPQ01 TAPQ03 TAPQ05 TAPQ07 
                   TAPQ08 TAPQ13 TAPQ14 TAPQ15
                   TAPQ16);

 * -------------------------------------------------------------- ;
 * --- EXCLUDE TRANSFERS ---------------------------------------- ;
 * -------------------------------------------------------------- ;

 * --- TRANSFER FROM ANOTHER ---------------- ;
 IF (2 <= ASOURCE <= 3) THEN DO;
   TAPQ01 = .;
   TAPQ02 = .;
   TAPQ03 = .;
   TAPQ05 = .;
   TAPQ07 = .;
   TAPQ08 = .;
   TAPQ09 = .;
   TAPQ10 = .;
   TAPQ11 = .;
   TAPQ12 = .;
   TAPQ13 = .;
   TAPQ14 = .;
   TAPQ15 = .;
   TAPQ16 = .;
   TAPQ90 = .;
   TAPQ91 = .;
   TAPQ92 = .;
 END;

RUN;

* --------------------------------- ;
PROC MEANS DATA=OUT1.&OUTFILA1. N NMISS MIN MAX MEAN SUM;
RUN;

PROC CONTENTS DATA=OUT1.&OUTFILA1. POSITION;
RUN;

PROC PRINT DATA=OUT1.&OUTFILA1.(OBS=24);
TITLE4 "FIRST 24 RECORDS IN OUTPUT DATA SET &OUTFILA1.";
RUN;

