*==================================================================
*  Title:  PROGRAM A2 CALCULATES OBSERVED AREA RATES FOR AHRQ
*          PREVENTION QUALITY INDICATORS
*
*  Description:
*         USES PROC SUMMARY TO CALCULATE OBSERVED RATES FOR
*         PREVENTION QUALITY INDICATORS ACROSS STRATIFIERS:
*         PROGRAM USES AREA, AGECAT, SEXCAT AND RACECAT
*
*          >>>  VERSION 3.2, March 2008 <<<
*
*===================================================================;
FILENAME CONTROL 'C:\PATHNAME\CONTROL_PQI.SAS'; *<===USER MUST modify;

%INCLUDE CONTROL;

 %MACRO CTY2MA;
    %IF &MALEVL EQ 0 %THEN %DO;
        ATTRIB MAREA LENGTH=$5
          LABEL='FIPS STATE COUNTY CODE';
        MAREA = FIPSTCO;
    %END;
    %ELSE %IF &MALEVL EQ 1 %THEN %DO;
        ATTRIB MAREA LENGTH=$5
          LABEL='MODIFIED FIPS';
        MAREA = INPUT(PUT(FIPSTCO,$M1AREA.),$5.);
    %END;
    %ELSE %IF &MALEVL EQ 2 %THEN %DO;
        ATTRIB MAREA LENGTH=$5
          LABEL='OMB 1999 METRO AREA';
        MAREA = INPUT(PUT(FIPSTCO,$M2AREA.),$5.);
    %END;
    %ELSE %IF &MALEVL EQ 3 %THEN %DO;
        ATTRIB MAREA LENGTH=$5
          LABEL='OMB 2003 METRO AREA';
        MAREA = INPUT(PUT(FIPSTCO,$M3AREA.),$5.);
    %END;
 %MEND;

 TITLE2 'PROGRAM A2';
 TITLE3 'AHRQ PREVENTION QUALITY INDICATORS: CALCULATE OBSERVED AREA RATES';
 run;

 * -------------------------------------------------------------- ;
 * --- Calculate the Numerators (T prefix) for the observed rates ;
 * --- and the Denominators (P prefix) for those Prevention       ;
 * --- Quality Indicators that are based on the hospitalized      ;
 * --- population.                                                ;
 * -------------------------------------------------------------- ;

 PROC   SUMMARY DATA=IN2.&INFILEA1. NWAY;
 CLASS  MAREA AGECAT SEXCAT RACECAT;
 VAR    TAPQ01-TAPQ03 TAPQ05 TAPQ07-TAPQ16 TAPQ90-TAPQ92;
 OUTPUT OUT=TEMP1
        SUM(TAPQ01-TAPQ03 TAPQ05 TAPQ07-TAPQ16 TAPQ90-TAPQ92)=
            TAPQ01-TAPQ03 TAPQ05 TAPQ07-TAPQ16 TAPQ90-TAPQ92
          N(TAPQ02 TAPQ09)=PAPQ02 PAPQ09 ;
 RUN;

 PROC   SORT DATA=TEMP1
                  (KEEP=MAREA AGECAT SEXCAT RACECAT
                        TAPQ01-TAPQ03 TAPQ05 TAPQ07-TAPQ16 TAPQ90-TAPQ92
                        PAPQ02 PAPQ09 _TYPE_);
 BY     MAREA AGECAT SEXCAT RACECAT;
 RUN;

 * -------------------------------------------------------------- ;
 * --- ADD THE AREA POPULATION DENOMINATORS --------------------- ;
 * -------------------------------------------------------------- ;

 PROC   SORT DATA=TEMP1 (KEEP=MAREA) OUT=MAREA NODUPKEY;
 BY     MAREA;
 RUN;
 
 * -------------------------------------------------------------- ;
 * --- READ CTY LEVEL POPS AND IF MALEVL SET TO 1,2,3 SUMMARIZE - ;
 * --- TO METRO AREA, ELSE SUMMARIZE TO CTY --------------------- ;
 * -------------------------------------------------------------- ;

 DATA QIPOP0;
    LENGTH FIPSTCO $5 SEXCAT POPCAT AGECAT RACECAT 3 
           POP_1995 POP_1996 POP_1997 POP_1998 POP_1999
           POP_2000 POP_2001 POP_2002 POP_2003 POP_2004
           POP_2005 POP_2006 POP_2007 POP_2008 POP 8;

    INFILE POPFILE MISSOVER FIRSTOBS=2;

    INPUT FIPSTCO SEXCAT POPCAT RACECAT 
          POP_1995 POP_1996 POP_1997 POP_1998 POP_1999
          POP_2000 POP_2001 POP_2002 POP_2003 POP_2004
          POP_2005 POP_2006 POP_2007 POP_2008;

    %CTY2MA

    IF POPCAT IN (1,2,3,4)            THEN AGECAT = 0;
    ELSE IF POPCAT IN (5,6,7,8)       THEN AGECAT = 1;
    ELSE IF POPCAT IN (9,10,11,12,13) THEN AGECAT = 2;
    ELSE IF POPCAT IN (14,15)         THEN AGECAT = 3;
    ELSE                                   AGECAT = 4;

    POP = POP_&POPYEAR.;

 RUN;

 PROC   SUMMARY DATA=QIPOP0 NWAY;
 CLASS  MAREA AGECAT SEXCAT RACECAT;
 VAR    POP;
 OUTPUT OUT=QIPOP SUM(POP)=;
 RUN;

 PROC   SORT DATA=QIPOP;
 BY     MAREA AGECAT SEXCAT RACECAT;
 RUN;

 DATA   QIPOP(KEEP=MAREA AGECAT SEXCAT RACECAT POP);
 MERGE  MAREA(IN=X) QIPOP;
 BY     MAREA;
 IF X;
 RUN;

 * -------------------------------------------------------------- ;
 * --- CALCULATE OBSERVED RATES                               --- ;
 * --- NOTE THAT OBSERVED RATES OF THE TOTAL AREA MAY OR MAY  --- ;
 * --- NOT BE DIFFERENT DEPENDING ON WHETHER COUNTY-LEVEL OR  --- ;
 * --- MA-LEVEL AREA STRATIFICATION WAS SPECIFIED.  THIS IS   --- ;
 * --- DUE TO THE INCLUSION OF COUNTY POPULATIONS UNDER MA-   --- ;
 * --- LEVEL STRATIFICATION THAT WOULD OTHERWISE BE EXCLUDED  --- ;
 * --- UNDER COUNTY-LEVEL STRATIFICATION.  UNDER COUNTY-LEVEL --- ;
 * --- STRATIFICATION, A COUNTY WOULD BE INCLUDED ONLY IF THERE - ;
 * --- WAS AT LEAST ONE DISCHARGE FROM THAT COUNTY.  UNDER MA --- ;
 * --- LEVEL STRATIFICATION, A COUNTY WOULD BE INCLUDED IF ANY -- ;
 * --- COUNTY WITHIN THE SAME MA HAD AT LEAST ONE DISCHARGE.   ---;
 *                                                                ;
 * --- DISCHARGES WITH MISSING PSTCO DATA HAVE BEEN           --- ;
 * --- EXCLUDED FROM THESE ANALYSIS.                          --- ;
 * -------------------------------------------------------------- ;
 
 DATA  TEMP1;
 MERGE TEMP1 QIPOP;
 BY    MAREA AGECAT SEXCAT RACECAT;

 IF POP = . THEN
    PUT 'MISSING POPULATION ' MAREA AGECAT SEXCAT RACECAT POP;

 ARRAY PPQ1{15} PAPQ01 PAPQ03 PAPQ05 PAPQ07 PAPQ08 PAPQ10-PAPQ16
                PAPQ90-PAPQ92;

 DO J = 1 to 15;
    PPQ1{J} = POP;
 END;

 IF AGECAT IN (0) THEN DO;
   TAPQ01 = .; PAPQ01 = .;
   TAPQ02 = .; PAPQ02 = .;
   TAPQ03 = .; PAPQ03 = .;
   TAPQ05 = .; PAPQ05 = .;
   TAPQ07 = .; PAPQ07 = .;
   TAPQ08 = .; PAPQ08 = .;
   TAPQ10 = .; PAPQ10 = .;
   TAPQ11 = .; PAPQ11 = .;
   TAPQ12 = .; PAPQ12 = .;
   TAPQ13 = .; PAPQ13 = .;
   TAPQ14 = .; PAPQ14 = .;
   TAPQ15 = .; PAPQ15 = .;
   TAPQ16 = .; PAPQ16 = .;
   TAPQ90 = .; PAPQ90 = .;
   TAPQ91 = .; PAPQ91 = .;
   TAPQ92 = .; PAPQ92 = .;
 END;
 IF AGECAT IN (1,2,3,4) THEN DO;
   TAPQ09 = .; PAPQ09 = .;
 END;

 RUN;

 * -------------------------------------------------------------- ;
 * --- AGGREGATE POPULATION COUNTS, BY STRATIFIERS            --- ;
 * -------------------------------------------------------------- ;

 PROC   SUMMARY DATA=TEMP1;
 CLASS  MAREA AGECAT SEXCAT RACECAT;
 VAR    TAPQ01-TAPQ03 TAPQ05 TAPQ07-TAPQ16 TAPQ90-TAPQ92
        PAPQ01-PAPQ03 PAPQ05 PAPQ07-PAPQ16 PAPQ90-PAPQ92;
 OUTPUT OUT=TEMP2 SUM=;
 RUN;

 DATA &OUTFILA2.;
 SET TEMP2;

 IF _TYPE_ &TYPELEVL;

 ARRAY PPQ2{17} PAPQ01-PAPQ03 PAPQ05 PAPQ07-PAPQ16 PAPQ90-PAPQ92;
 ARRAY TPQ{17} TAPQ01-TAPQ03 TAPQ05 TAPQ07-TAPQ16 TAPQ90-TAPQ92;
 ARRAY OPQ{17} OAPQ01-OAPQ03 OAPQ05 OAPQ07-OAPQ16 OAPQ90-OAPQ92;

 DO J = 1 to 17;
    IF TPQ{J} GT 0 AND PPQ2{J} GT 0 THEN OPQ{J} = TPQ{J} / PPQ2{J};
    ELSE IF PPQ2{J} GT 0 THEN OPQ{J} = 0 ;
 END;

 LABEL
 TAPQ01 = 'DIABETES SHORT TRM COMPLICATN (Numeratr)'
 TAPQ02 = 'PERFORATED APPENDIX           (Numeratr)'
 TAPQ03 = 'DIABETES LONG TERM COMPLICATN (Numeratr)'
 TAPQ05 = 'COPD                          (Numeratr)'
 TAPQ07 = 'HYPERTENSION                  (Numeratr)'
 TAPQ08 = 'CONGESTIVE HEART FAILURE      (Numeratr)'
 TAPQ09 = 'LOW BIRTH WEIGHT              (Numeratr)'
 TAPQ10 = 'DEHYDRATION                   (Numeratr)'
 TAPQ11 = 'BACTERIAL PNEUMONIA           (Numeratr)'
 TAPQ12 = 'URINARY INFECTION             (Numeratr)'
 TAPQ13 = 'ANGINA                        (Numeratr)'
 TAPQ14 = 'DIABETES UNCONTROLLED         (Numeratr)'
 TAPQ15 = 'ADULT ASTHMA                  (Numeratr)'
 TAPQ16 = 'LOWER EXTREMITY AMPUTATION    (Numeratr)'
 TAPQ90 = 'OVERALL PQI                   (Numeratr)'
 TAPQ91 = 'ACUTE PQI                     (Numeratr)'
 TAPQ92 = 'CHRONIC PQI                   (Numeratr)'
 ;
 LABEL
 PAPQ01 = 'DIABETES SHORT TRM CMPLICTN(Denominator)'
 PAPQ02 = 'PERFORATED APPENDIX        (Denominator)'
 PAPQ03 = 'DIABETES LONG TRM CMPLICTN (Denominator)'
 PAPQ05 = 'COPD                       (Denominator)'
 PAPQ07 = 'HYPERTENSION               (Denominator)'
 PAPQ08 = 'CONGESTIVE HEART FAILURE   (Denominator)'
 PAPQ09 = 'LOW BIRTH WEIGHT           (Denominator)'
 PAPQ10 = 'DEHYDRATION                (Denominator)'
 PAPQ11 = 'BACTERIAL PNEUMONIA        (Denominator)'
 PAPQ12 = 'URINARY INFECTION          (Denominator)'
 PAPQ13 = 'ANGINA                     (Denominator)'
 PAPQ14 = 'DIABETES UNCONTROLLED      (Denominator)'
 PAPQ15 = 'ADULT ASTHMA               (Denominator)'
 PAPQ16 = 'LOWER EXTREMITY AMPUTATION (Denominator)'
 PAPQ90 = 'OVERALL PQI                (Denominator)'
 PAPQ91 = 'ACUTE PQI                  (Denominator)'
 PAPQ92 = 'CHRONIC PQI                (Denominator)'
 ;
 LABEL
 OAPQ01 = 'DIABETES SHORT TRM COMPLICATN (Observed)'
 OAPQ02 = 'PERFORATED APPENDIX           (Observed)'
 OAPQ03 = 'DIABETES LONG TERM COMPLICATN (Observed)'
 OAPQ05 = 'COPD                          (Observed)'
 OAPQ07 = 'HYPERTENSION                  (Observed)'
 OAPQ08 = 'CONGESTIVE HEART FAILURE      (Observed)'
 OAPQ09 = 'LOW BIRTH WEIGHT              (Observed)'
 OAPQ10 = 'DEHYDRATION                   (Observed)'
 OAPQ11 = 'BACTERIAL PNEUMONIA           (Observed)'
 OAPQ12 = 'URINARY INFECTION             (Observed)'
 OAPQ13 = 'ANGINA                        (Observed)'
 OAPQ14 = 'DIABETES UNCONTROLLED         (Observed)'
 OAPQ15 = 'ADULT ASTHMA                  (Observed)'
 OAPQ16 = 'LOWER EXTREMITY AMPUTATION    (Observed)'
 OAPQ90 = 'OVERALL PQI                   (Observed)'
 OAPQ91 = 'ACUTE PQI                     (Observed)'
 OAPQ92 = 'CHRONIC PQI                   (Observed)'
 _TYPE_ = 'Stratification level                    '
 ;

 DROP J;

 RUN;

 PROC SORT DATA=&OUTFILA2. OUT=OUT2.&OUTFILA2.;
 BY MAREA AGECAT SEXCAT RACECAT;
 RUN;

 * -------------------------------------------------------------- ;
 * --- CONTENTS AND MEANS OF AREA OBSERVED MEANS FILE ----------- ;
 * -------------------------------------------------------------- ;

 PROC CONTENTS DATA=OUT2.&OUTFILA2. POSITION;
 RUN;

 PROC MEANS DATA=OUT2.&OUTFILA2.(WHERE=(_TYPE_ IN (8))) N NMISS MIN MAX MEAN SUM;
 TITLE4 "SUMMARY OF AREA-LEVEL RATES (_TYPE_=8)";
 RUN;

 * -------------------------------------------------------------- ;
 * --- PRINT AREA OBSERVED MEANS FILE --------------------------- ;
 * -------------------------------------------------------------- ;

 %MACRO PRT2;

 %IF &PRINT. = 1 %THEN %DO;

 %MACRO PRT(PQ,TEXT);

 PROC  PRINT DATA=OUT2.&OUTFILA2. LABEL SPLIT='*';
 VAR   MAREA AGECAT SEXCAT RACECAT TAPQ&PQ. PAPQ&PQ. OAPQ&PQ. ;
 LABEL MAREA   = "MAREA"
       AGECAT  = "AGECAT"
       SEXCAT  = "SEXCAT"
       RACECAT = "RACECAT"
       TAPQ&PQ.   = "TAPQ&PQ.*(Numerator)"
       PAPQ&PQ.   = "PAPQ&PQ.*(Denominator)"
       OAPQ&PQ.   = "OAPQ&PQ.*(Observed)"
       ;
 FORMAT AGECAT AGECAT.   
        SEXCAT SEXCAT.
        RACECAT RACECAT.
	  TAPQ&PQ. PAPQ&PQ. COMMA13.0
	  OAPQ&PQ. 8.6
        ;
 TITLE4 "Indicator # &PQ.: &TEXT";
 RUN;

 %MEND PRT;

 %PRT(01,DIABETES SHORT TERM CX);
 %PRT(02,PERFORATED APPENDIX);
 %PRT(03,DIABETES LONG TERM CX);
 %PRT(05,COPD);
 %PRT(07,HYPERTENSION);
 %PRT(08,CHF);
 %PRT(09,LOW BIRTH WEIGHT);
 %PRT(10,DEHYDRATION);
 %PRT(11,BACTERIAL PNEUMONIA);
 %PRT(12,URINARY INFECTION);
 %PRT(13,ANGINA W/O PROCEDURES);
 %PRT(14,DIABETES UNCONTROLLED);
 %PRT(15,ADULT ASTHMA);
 %PRT(16,LOWER EXTREMITY AMPUTATION DM);
 %PRT(90,OVERALL PQI);
 %PRT(91,ACUTE PQI);
 %PRT(92,CHRONIC PQI);

 %END;

 %MEND PRT2;

 %PRT2; 

 * -------------------------------------------------------------- ;
 * --- WRITE SAS OUTPUT DATA SET TO COMMA-DELIMITED TEXT FILE --- ;
 * --- FOR EXPORT INTO SPREADSHEETS ----------------------------- ;
 * -------------------------------------------------------------- ;

 %MACRO TEXT;

 %IF &TEXTA2. = 1  %THEN %DO; 

 DATA _NULL_;
 SET OUT2.&OUTFILA2;
 FILE PQTEXTA2 LRECL=1000;
 IF _N_=1 THEN PUT "MAREA" "," "Age" "," "Sex" "," "Race" "," "Type" ","
 "TAPQ01" "," "TAPQ02" "," "TAPQ03" "," 
 "TAPQ05" "," "TAPQ07" "," "TAPQ08" ","
 "TAPQ09" "," "TAPQ10" "," "TAPQ11" "," "TAPQ12" "," 
 "TAPQ13" "," "TAPQ14" "," "TAPQ15" "," "TAPQ16" ","
 "TAPQ90" "," "TAPQ91" "," "TAPQ92" ","
 "PAPQ01" "," "PAPQ02" "," "PAPQ03" "," 
 "PAPQ05" "," "PAPQ07" "," "PAPQ08" ","
 "PAPQ09" "," "PAPQ10" "," "PAPQ11" "," "PAPQ12" "," 
 "PAPQ13" "," "PAPQ14" "," "PAPQ15" "," "PAPQ16" ","
 "PAPQ90" "," "PAPQ91" "," "PAPQ92" ","
 "OAPQ01" "," "OAPQ02" "," "OAPQ03" "," 
 "OAPQ05" "," "OAPQ07" "," "OAPQ08" ","
 "OAPQ09" "," "OAPQ10" "," "OAPQ11" "," "OAPQ12" "," 
 "OAPQ13" "," "OAPQ14" "," "OAPQ15" "," "OAPQ16" ","
 "OAPQ90" "," "OAPQ91" "," "OAPQ92";

 PUT MAREA $5. "," AGECAT 3. "," SEXCAT 3. "," RACECAT 3. "," _TYPE_ 2. ","
 (TAPQ01-TAPQ03 TAPQ05 TAPQ07-TAPQ16 TAPQ90-TAPQ92) (7.0 ",")
 ","
 (PAPQ01-PAPQ03 PAPQ05 PAPQ07-PAPQ16 PAPQ90-PAPQ92) (13.2 ",") 
 ","
 (OAPQ01-OAPQ03 OAPQ05 OAPQ07-OAPQ16 OAPQ90-OAPQ92) (12.10 ",")
 ;
 RUN;

 %END;

 %MEND TEXT;

 %TEXT;

